#include "stdafx.h"
#include "index.h"
#include "TextManager.h"

namespace nsAbyssLib{
namespace nsTextManager{

//ftHgRXgN^
TextManager::TextManager(){
	Initialize();
	SetForceSkip(false);
	SetAutoModeTime(DEFAULT_TEXT_AUTOMODETIME);
	SetKidokuMaxLog(DEFAULT_KIDOKU_MAXLOG);
}

//
void TextManager::Initialize(){
	SetSkip(false);
	SetAutoMode(false);
	SetReadIterator(1);
	ClearRecordData();
}

//XLbv擾
bool TextManager::GetSkip() const{
	return(m_IsSkip);
}
//XLbvݒ
void TextManager::SetSkip(bool IsSkip){
	m_IsSkip = IsSkip;
}

//XLbv擾
bool TextManager::GetForceSkip() const{
	return(m_ForceSkip);
}
//XLbvݒ
void TextManager::SetForceSkip(bool IsForceSkip){
	m_ForceSkip = IsForceSkip;
}

//I[g[h̗L擾
bool TextManager::GetAutoMode() const{
	return(m_IsAutoMode);
}
//I[g[h̗Lݒ
void TextManager::SetAutoMode(bool IsAutoMode){
	m_IsAutoMode = IsAutoMode;
}

//I[g[h̑ҋ@Ԃ擾
unsigned int TextManager::GetAutoModeTime() const{
	return(m_AutoModeTime);
}
//I[g[h̑ҋ@Ԃݒ
void TextManager::SetAutoModeTime(unsigned int Time){
	m_AutoModeTime = Time;
}

//O̍őێ擾
unsigned int TextManager::GetKidokuMaxLog() const{
	return(m_KidokuMaxLog);
}
//O̍őێݒ
void TextManager::SetKidokuMaxLog(unsigned int NumMaxLog){
	m_KidokuMaxLog = NumMaxLog;
}

//L^ɒǉ
void TextManager::AddRecord(unsigned int Iterator){
	if(!IsExistRecord(Iterator)){
		m_Record.push_back(Iterator);
	}
}
//ǂ𔻒
bool TextManager::IsExistRecord(unsigned int Iterator) const{
	for(unsigned int i=0;i<m_Record.size();i++){
		if(Iterator==m_Record.at(i)){
			return(true);
		}
	}
	return(false);
}

//Ce[^
void TextManager::AddReadIterator(int Iterator){
	m_ReadIterator += Iterator;
	if(m_ReadIterator<1){
		m_ReadIterator = 1;
	}
	if(m_ReadIterator>m_RecordData.size()){
		m_ReadIterator = static_cast<unsigned int>(m_RecordData.size());
	}
}
//Ce[^擾
unsigned int TextManager::GetReadIterator() const{
	return(m_ReadIterator);
}
//Ce[^ݒ
void TextManager::SetReadIterator(unsigned int Iterator){
	m_ReadIterator = Iterator;
	if(m_ReadIterator<1){
		m_ReadIterator = 1;
	}
	if(m_ReadIterator>m_RecordData.size()){
		m_ReadIterator = static_cast<unsigned int>(m_RecordData.size());
	}
}

//O
void TextManager::ClearRecordData(){
	m_RecordData.clear();
}
//O󂩁H
bool TextManager::IsEmptyRecordData() const{
	return(m_RecordData.empty());
}
//O擾
const Record& TextManager::GetRecordData() const{
	if(!IsEmptyRecordData()){
		return(m_RecordData.at(m_RecordData.size()-m_ReadIterator));
	}
	else{
		RuntimeError(NULL, "TextManager::GetRecordData()");
	}
}
//Oǉ
void TextManager::AddRecordData(const Record& Data){
	m_RecordData.push_back(Data);
	while(m_RecordData.size()>GetKidokuMaxLog()){
		m_RecordData.erase(m_RecordData.begin());
	}
}

//ۑiݒj
void TextManager::SaveE(IIOStream* IOStream){
	SaveTemplate(IOStream, m_ForceSkip);
	SaveTemplate(IOStream, m_AutoModeTime);
	SaveTemplate(IOStream, m_KidokuMaxLog);
}
//ǂݍ݁iݒj
void TextManager::LoadE(IIOStream* IOStream){
	LoadTemplate(IOStream, &m_ForceSkip);
	LoadTemplate(IOStream, &m_AutoModeTime);
	LoadTemplate(IOStream, &m_KidokuMaxLog);
}
//ۑiǋL^j
bool TextManager::SaveK(const string& FileName, IIOStream* IOStream){
	if(!IOStream->Open(FileName.c_str(), "wb")){
		return(false);
	}
	char Header[4] = {0};

	Header[0] = '.'; Header[1] = 'k'; Header[2] = 's'; Header[3] = 'v';
	IOStream->SavePrimitive(Header, sizeof(Header));
	SaveTemplate(IOStream, static_cast<int>(m_Record.size()));
	for(unsigned int i=0;i<m_Record.size();i++){
		SaveTemplate(IOStream, m_Record.at(i));
	}
	return(IOStream->Close());
}
//ǂݍ݁iǋL^j
bool TextManager::LoadK(const string& FileName, IIOStream* IOStream){
	if(!IOStream->Open(FileName.c_str(), "rb")){
		return(false);
	}
	char Header[4] = {0};
	IOStream->LoadPrimitive(Header, sizeof(Header));
	if(Header[0]=='.' && Header[1]=='k' && Header[2]=='s' && Header[3]=='v'){
		int Loop;
		ClearRecordData();
		LoadTemplate(IOStream, &Loop);
		for(int i=0;i<Loop;i++){
			m_Record.push_back(0);
			LoadTemplate(IOStream, &m_Record.back());
		}
		return(IOStream->Close());
	}
	else{
		IOStream->Close();
		return(false);
	}
}
//ۑi[Jj
void TextManager::SaveL(IIOStream* IOStream){
	SaveTemplate(IOStream, static_cast<int>(m_RecordData.size()));
	for(vector<Record>::iterator it=m_RecordData.begin();it!=m_RecordData.end();it++){
		IOStream->SaveRecord(*it);
	}
}
//ǂݍ݁i[Jj
void TextManager::LoadL(IIOStream* IOStream){
	SetSkip(false);
	SetAutoMode(false);
	int Loop;
	ClearRecordData();
	LoadTemplate(IOStream, &Loop);
	for(int i=0;i<Loop;i++){
		m_RecordData.push_back(Record());
		IOStream->LoadRecord(&m_RecordData.back());
	}
}

}
}